(* ******************************************************************************************


  AMPRNG URPN - I'M PRNG UNIQUE RANDOM PIXEL NUMBER GENERATOR
  Pseudo-random probably safe pixel number generator

  Copyright (C) 2011  Alexander Myasnikow

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.


  ******************************************************************************************* *)

unit uurpn;

interface

uses amprng;

Type
  TURPN = Class(TAMPRNG)

  private
    sX, sY: integer;
    lXY: integer;
    pMax: integer;
    pCount: integer;
    pUsed: array of boolean;
    byX: boolean;

  public
    constructor Create(const password: string; x, y: integer); overload;
    destructor Destroy; overload;
    function RandomXY(var x: integer; var y: integer; var abs: integer): boolean;
    function RandomABS (var abs: integer): boolean;
  End;

implementation

destructor TURPN.Destroy;
begin
  SetLength(pUsed, 0);
  inherited Destroy;
end;

function TURPN.RandomABS(var abs: Integer): boolean;
var x,y : integer;
begin

Result := RandomXY(x, y, abs);

end;

constructor TURPN.Create(const password: string; x: integer; y: integer);
begin
  pCount := 0;
  sX := x;
  sY := y;
  pMax := x * y;
  SetLength(pUsed, pMax);

  if x > y then
  begin
    lXY := x;
    byX := true;
  end
  else
  begin
    lXY := y;
    byX := false;
  end;

  FillChar(pUsed[0], pMax, 0);

  inherited Create(password, 64, 2048);
end;

function TURPN.RandomXY(var x: integer; var y: integer; var abs: integer): boolean;
var
  rv: integer;
begin

  if pCount = pMax then
  begin
    Result := false;
    exit;
  end;

  repeat
    rv := self.Random(pMax);
  until not pUsed[rv];

  pUsed[rv] := true;

  if byX then
  begin
    x := rv mod lXY;
    y := rv div lXY;
    abs := rv;
  end
  else
  begin
    x := rv div lXY;
    y := rv mod lXY;
    abs := rv;
  end;
  inc(pCount);

  Result := true;
end;

end.
